/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

//Initialize Global Variables

delayTime = [3.156, 3.156, 3.156, 3.156] //Set default to register default value

delayTime_Used = 0; //Set defaut to 0, will get overwritten

is20Bit = -1; //default to unused value in case of error
isSPI = -1; //default to unused value in case of error

numSamplesCollected = 0;

const PLOT_VSHUNT   = 0b1000000;
const PLOT_VBUS     = 0b0100000;
const PLOT_DIETEMP  = 0b0010000;
const PLOT_CURRENT  = 0b0001000;
const PLOT_POWER    = 0b0000100;
const PLOT_ENERGY   = 0b0000010;
const PLOT_CHARGE   = 0b0000001;

const MEASURE_VBUS      = 0b0001;
const MEASURE_VSHUNT    = 0b0010;
const MEASURE_TEMP      = 0b0100;

let saveText = function(fname, text) {
    let blob = new Blob(['\ufeff' + text], {  //'\ufeff' added for excel printout when using special characters
        type: 'text/csv;charset=utf-8'
    });
    saveAs(blob, fname);

    return false;
}

function isEmptyOrSpaces(str)
{
    return str === null || (/^\s*$/).test(str);
}

function roundTo(n, digits) 
{
    if (digits === undefined) {
        digits = 0;
    }

    var multiplicator = Math.pow(10, digits);
    n = parseFloat((n * multiplicator).toFixed(11));
    return Math.round(n) / multiplicator;
}

function savePlots(plotFlags, fileName)
{
    let numEVMs = templateObj.$.ti_widget_droplist_numEVMs.selectedValue;
    let strDataArray = [];
    let result = "Sample";
    
    if (plotFlags & PLOT_VSHUNT)
    {
        result = result + ",EVM1 VSHUNT Results (mV)";
        strDataArray.push(templateObj.$.ti_widget_scalargraph_VSHUNT.series_0_values);
        if(numEVMs > 1)
        {
            result = result + ",EVM2 VSHUNT Results (mV)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_VSHUNT.series_1_values);
        }
        if(numEVMs > 2)
        {
            result = result + ",EVM3 VSHUNT Results (mV)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_VSHUNT.series_2_values);
        }
        if(numEVMs > 3)
        {
            result = result + ",EVM4 VSHUNT Results (mV)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_VSHUNT.series_3_values);
        }
    }
    if (plotFlags & PLOT_VBUS)
    {
        result = result + ",EVM1 VBUS Results (V)";
        strDataArray.push(templateObj.$.ti_widget_scalargraph_VBUS.series_0_values);
        if(numEVMs > 1)
        {
            result = result + ",EVM2 VBUS Results (V)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_VBUS.series_1_values);
        }
        if(numEVMs > 2)
        {
            result = result + ",EVM3 VBUS Results (V)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_VBUS.series_2_values);
        }
        if(numEVMs > 3)
        {
            result = result + ",EVM4 VBUS Results (V)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_VBUS.series_3_values);
        }
    }
    if (plotFlags & PLOT_DIETEMP)
    {
        let units
        if(templateObj.$.ti_widget_droplist_dietemp_unit.selectedValue === 2)
        {
            units = " (°F)";
        }
        else
        {
            units = " (°C)";
        }
        result = result + ",EVM1 DIETEMP Results" + units;
        strDataArray.push(templateObj.$.ti_widget_scalargraph_DIETEMP.series_0_values);
        if(numEVMs > 1)
        {
            result = result + ",EVM2 DIETEMP Results" + units;
            strDataArray.push(templateObj.$.ti_widget_scalargraph_DIETEMP.series_1_values);
        }
        if(numEVMs > 2)
        {
            result = result + ",EVM3 DIETEMP Results" + units;
            strDataArray.push(templateObj.$.ti_widget_scalargraph_DIETEMP.series_2_values);
        }
        if(numEVMs > 3)
        {
            result = result + ",EVM4 DIETEMP Results" + units;
            strDataArray.push(templateObj.$.ti_widget_scalargraph_DIETEMP.series_3_values);
        }
    }
    if (plotFlags & PLOT_CURRENT)
    {
        result = result + ",EVM1 CURRENT Results (A)";
        strDataArray.push(templateObj.$.ti_widget_scalargraph_CURRENT.series_0_values);
        if(numEVMs > 1)
        {
            result = result + ",EVM2 CURRENT Results (A)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_CURRENT.series_1_values);
        }
        if(numEVMs > 2)
        {
            result = result + ",EVM3 CURRENT Results (A)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_CURRENT.series_2_values);
        }
        if(numEVMs > 3)
        {
            result = result + ",EVM4 CURRENT Results (A)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_CURRENT.series_3_values);
        }
    }
    if (plotFlags & PLOT_POWER)
    {
        result = result + ",EVM1 POWER Results (W)";
        strDataArray.push(templateObj.$.ti_widget_scalargraph_POWER.series_0_values);
        if(numEVMs > 1)
        {
            result = result + ",EVM2 POWER Results (W)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_POWER.series_1_values);
        }
        if(numEVMs > 2)
        {
            result = result + ",EVM3 POWER Results (W)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_POWER.series_2_values);
        }
        if(numEVMs > 3)
        {
            result = result + ",EVM4 POWER Results (W)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_POWER.series_3_values);
        }
    }
    if (plotFlags & PLOT_ENERGY)
    {
        result = result + ",EVM1 ENERGY Results (J)";
        strDataArray.push(templateObj.$.ti_widget_scalargraph_ENERGY.series_0_values);
        if(numEVMs > 1)
        {
            result = result + ",EVM2 ENERGY Results (J)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_ENERGY.series_1_values);
        }
        if(numEVMs > 2)
        {
            result = result + ",EVM3 ENERGY Results (J)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_ENERGY.series_2_values);
        }
        if(numEVMs > 3)
        {
            result = result + ",EVM4 ENERGY Results (J)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_ENERGY.series_3_values);
        }
    }
    if (plotFlags & PLOT_CHARGE)
    {
        result = result + ",EVM1 CHARGE Results (C)";
        strDataArray.push(templateObj.$.ti_widget_scalargraph_CHARGE.series_0_values);
        if(numEVMs > 1)
        {
            result = result + ",EVM2 CHARGE Results (C)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_CHARGE.series_1_values);
        }
        if(numEVMs > 2)
        {
            result = result + ",EVM3 CHARGE Results (C)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_CHARGE.series_2_values);
        }
        if(numEVMs > 3)
        {
            result = result + ",EVM4 CHARGE Results (C)";
            strDataArray.push(templateObj.$.ti_widget_scalargraph_CHARGE.series_3_values);
        }
    }
    result = result + "\n";
    
    for (let i = 0; i < strDataArray[0].length; i++)
    {
        result = result + i;
        
        for (let j = 0; j < strDataArray.length; j++) 
        {
            result = result + "," + strDataArray[j][i];
        }
        result = result + "\n";
    }
    
    saveText(fileName, result);
}


/*
*  Boilerplate code for creating computed data bindings
*/
document.addEventListener('gc-databind-ready', function() 
{
    /* 
	*   Add custom computed value databindings here, using the following method:
	*
    *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
	*
	*   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
	*   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
    */
	
	// For example, a simple computed values based on simple expression
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");
	
	// Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });
	
	// Event 1 to n bindings
	/* 
    gc.databind.registry.bind('widget.date.value', 
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        }, 
        // getter for date computation
        function(values) 
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    ); 
	*/
	
	//bind results registers to plots
    // gc.databind.registry.bind('widget.ti_widget_scalargraph_VSHUNT.value_0', 'ti_model_register.vshunt_register.vshunt',
    //     function(value) 
    //     {
    //         let result = value;
    //         return result;
    //     }
    // ); 
    // gc.databind.registry.bind('widget.ti_widget_scalargraph_VBUS.value_0', 'ti_model_register.vbus_register.vbus',
    //     function(value) 
    //     {
    //         let result = value;
    //         return result;
    //     }
    // ); 
    // gc.databind.registry.bind('widget.ti_widget_scalargraph_DIETEMP.value_0', 'ti_model_register.dietemp_register.dietemp',
    //     function(value) 
    //     {
    //         let result = value;
    //         return result;
    //     }
    // ); 
    // gc.databind.registry.bind('widget.ti_widget_scalargraph_CURRENT.value_0', 'ti_model_register.current_register.current',
    //     function(value) 
    //     {
    //         let result = value;
    //         return result;
    //     }
    // ); 
    // gc.databind.registry.bind('widget.ti_widget_scalargraph_POWER.value_0', 'ti_model_register.power_register.power',
    //     function(value) 
    //     {
    //         let result = value;
    //         return result;
    //     }
    // ); 
    // gc.databind.registry.bind('widget.ti_widget_scalargraph_ENERGY.value_0', 'ti_model_register.energy_register.energy',
    //     function(value) 
    //     {
    //         let result = value;
    //         return result;
    //     }
    // ); 
    // gc.databind.registry.bind('widget.ti_widget_scalargraph_CHARGE.value_0', 'ti_model_register.charge_register.charge',
    //     function(value) 
    //     {
    //         let result = value;
    //         return result;
    //     }
    // ); 
    
    // Bind settings to specific devices
 	gc.databind.registry.bind('widget.ti_widget_checkbox_ENERGY.checked', 'widget.ti_model_register.$selectedDevice', 
     	function(deviceName)
     	{
            if(deviceName === "INA237_238" || deviceName === "INA239")
            {
                templateObj.$.ti_widget_checkbox_ENERGY.disabled = true;
                templateObj.$.ti_widget_checkbox_ENERGY.visible = false;
                //templateObj.$.ti_widget_container.style.height = "174px";
                return false;
            }
            templateObj.$.ti_widget_checkbox_ENERGY.disabled = false;
            templateObj.$.ti_widget_checkbox_ENERGY.visible = true;
            //templateObj.$.ti_widget_container.style.height = "220px";
            return true;
        }
	);
 	gc.databind.registry.bind('widget.ti_widget_checkbox_CHARGE.checked', 'widget.ti_model_register.$selectedDevice', 
     	function(deviceName)
     	{
            if(deviceName === "INA237_238" || deviceName === "INA239")
            {
                templateObj.$.ti_widget_checkbox_CHARGE.disabled = true;
                templateObj.$.ti_widget_checkbox_CHARGE.visible = false;
                return false;
            }
            
            templateObj.$.ti_widget_checkbox_CHARGE.disabled = false;
            templateObj.$.ti_widget_checkbox_CHARGE.visible = true;
            return true;
        }
	);
	gc.databind.registry.bind('widget.ti_widget_droplist_E1_SW1.labels', 'widget.ti_model_register.$selectedDevice', 
     	function(deviceName)
     	{
            if(deviceName === "INA229" || deviceName === "INA239")
            {
                templateObj.$.ti_widget_container_E1_SW0.visible = false;
                return "|CS1|CS2|CS3|CS4";
            }
            templateObj.$.ti_widget_container_E1_SW0.visible = true;
            return "|GND|VS|SDA|SCL";
        }
    );
    gc.databind.registry.bind('widget.ti_widget_droplist_E2_SW1.labels', 'widget.ti_model_register.$selectedDevice', 
     	function(deviceName)
     	{
            if(deviceName === "INA229" || deviceName === "INA239")
            {
                templateObj.$.ti_widget_container_E2_SW0.visible = false;
                return "|CS1|CS2|CS3|CS4";
            }
            templateObj.$.ti_widget_container_E2_SW0.visible = true;
            return "|GND|VS|SDA|SCL";
        }
	);
	gc.databind.registry.bind('widget.ti_widget_droplist_E3_SW1.labels', 'widget.ti_model_register.$selectedDevice', 
     	function(deviceName)
     	{
            if(deviceName === "INA229" || deviceName === "INA239")
            {
                templateObj.$.ti_widget_container_E3_SW0.visible = false;
                return "|CS1|CS2|CS3|CS4";
            }
            templateObj.$.ti_widget_container_E3_SW0.visible = true;
            return "|GND|VS|SDA|SCL";
        }
	);
	gc.databind.registry.bind('widget.ti_widget_droplist_E4_SW1.labels', 'widget.ti_model_register.$selectedDevice', 
     	function(deviceName)
     	{
            if(deviceName === "INA229" || deviceName === "INA239")
            {
                templateObj.$.ti_widget_container_E4_SW0.visible = false;
                return "|CS1|CS2|CS3|CS4";
            }
            templateObj.$.ti_widget_container_E4_SW0.visible = true;
            return "|GND|VS|SDA|SCL";
        }
	);
	gc.databind.registry.bind('widget.ti_widget_droplist_selectedEVM.labels', 'widget.ti_widget_droplist_numEVMs.selectedValue', 
     	function(numEVMs)
     	{
     	    if(templateObj.$.ti_widget_droplist_selectedEVM.selectedValue > (numEVMs-1))
     	    {
     	        templateObj.$.ti_widget_droplist_selectedEVM.selectedValue = 0;
     	    }
            if(numEVMs === 4)
            {
                templateObj.$.ti_widget_droplist_selectedEVM.values = "0,1,2,3";
                return "EVM 1|EVM 2|EVM 3|EVM 4";
            }
            else if(numEVMs === 3)
            {
                templateObj.$.ti_widget_droplist_selectedEVM.values = "0,1,2";
                 return "EVM 1|EVM 2|EVM 3";
            }
            else if(numEVMs === 2)
            {
                templateObj.$.ti_widget_droplist_selectedEVM.values = "0,1";
                 return "EVM 1|EVM 2";
            }
            else
            {
                templateObj.$.ti_widget_droplist_selectedEVM.values = "0";
                 return "EVM 1";
            }
            
        }
	);
	gc.databind.registry.bind('widget.ti_widget_label_lsbwarning.visible',
    {
        current: 'widget.ti_widget_checkbox_CURRENT.checked',
        power: 'widget.ti_widget_checkbox_POWER.checked',
        energy: 'widget.ti_widget_checkbox_ENERGY.checked',
        charge: 'widget.ti_widget_checkbox_CHARGE.checked',
        rshunt: 'widget.ti_widget_textbox_rshunt.value',
        shuntCal: 'widget.ti_widget_numberbox_shuntcal.value',
        numEVMs: 'widget.ti_widget_droplist_numEVMs.selectedValue',
    },
     	function(values)
     	{
     	    let warning = false;
     	    
            if(values.current || values.power || values.energy || values.charge)
            {
                for(let i = 0; i < values.numEVMs; i++)
                {
                    if(isEmptyOrSpaces(data[i].rshunt) || isEmptyOrSpaces(data[i].shuntcal))
                    {
                        warning = true;
                    }
                }
            }
            return warning;
        }
	);
    gc.databind.registry.bind('widget.ti_widget_numberbox_collect_delay.value',
    {
        mode: 'ti_model_register.adc_config_register.mode',
        vbusct: 'ti_model_register.adc_config_register.vbusct',
        vshct: 'ti_model_register.adc_config_register.vshct',
        vtct: 'ti_model_register.adc_config_register.vtct',
        avg: 'ti_model_register.adc_config_register.avg',
        numEVMs: 'widget.ti_widget_droplist_numEVMs.selectedValue',
        selectedevm: 'widget.ti_widget_droplist_selectedEVM.selectedValue', //added to update values when changing EVMs
        //autoDelay: 'ti_widget_checkbox_autodelay.checked',  //taken care of in event listeners due to issues here
    },
        function(values)
        {
            let delay;
            let conversionTimes = [50, 84, 150, 280, 540, 1052, 2074, 4120];
            let averages = [1, 4, 16, 64, 128, 256, 512, 1024];
            
            let calcDelay = 0;
            
            // if(!values.autoDelay) //not needed so that we can track the values even when not using them
            // {
            //     return templateObj.$.ti_widget_numberbox_collect_delay.value;
            // }
            
            if(values.mode & MEASURE_VBUS)
            {
                calcDelay = calcDelay + (conversionTimes[values.vbusct] / 1000);
            }
            if(values.mode & MEASURE_VSHUNT)
            {
                calcDelay = calcDelay + (conversionTimes[values.vshct] / 1000);
            }
            if(values.mode & MEASURE_TEMP)
            {
                calcDelay = calcDelay + (conversionTimes[values.vtct] / 1000);
            }
            
            calcDelay = calcDelay * averages[values.avg]
            
            delayTime[values.selectedevm] = calcDelay;
            
            delay = delayTime[0];
            
            for(let i = 1; i < values.numEVMs; i++)
            {
                delay = Math.min(delay, delayTime[i]);
            }
            
            delayTime_Used = delay;
            
            if (templateObj.$.ti_widget_checkbox_autodelay.checked && delay != templateObj.$.ti_widget_numberbox_collect_delay.value)
            {
                Utilities.fireEvent('update-delay', Math.round(delay*1000), "plots.js");
            }
            
            if(!templateObj.$.ti_widget_checkbox_autodelay.checked)
            {
                return templateObj.$.ti_widget_numberbox_collect_delay.value;
            }

            return delay;
	    }
    );
    
    gc.databind.registry.bind('widget.ti_widget_label_E1_star.visible',
    {
        selectedevm: 'widget.ti_widget_droplist_selectedEVM.selectedValue',
        numevms: 'widget.ti_widget_droplist_numEVMs.selectedValue',
    },
        function(values)
        {
            let visible = false;
            
            if(values.selectedevm === 0 && values.numevms !== 1)
            {
                visible = true;
            }
            return visible;
        }
	);
    
    
    //Change Listeners 
    
    
    // gc.databind.registry.getBinding('widget.ti_model_register.$selectedDevice').addChangedListener(  //done in scbCustomCodec
    // {
    //     onValueChanged: function(oldValue, newValue)
    //     {
    //         if(newValue === "INA229" || newValue === "INA239")
    //         {
    //             isSPI = true;
    //         }
    //         else
    //         {
    //             isSPI = false;
    //         }
    //         if(newValue === "INA229" || newValue === "INA228")
    //         {
    //             is20Bit = true;
    //         }
    //         else
    //         {
    //             is20Bit = false;
    //         }
    //  	}
    // });
    
    gc.databind.registry.getBinding('widget.ti_widget_droplist_selectedEVM.selectedValue').addChangedListener(
    {
        onValueChanged: function(oldValue, newValue)
        {
            Utilities.fireEvent('select-evm', templateObj.$.ti_widget_droplist_selectedEVM.selectedValue, "plots.js");
     	}
    });
    
    const swE1 = {
        onValueChanged: function() {
            data[0].sw0 = templateObj.$.ti_widget_droplist_E1_SW0.selectedValue;
            data[0].sw1 = templateObj.$.ti_widget_droplist_E1_SW1.selectedValue;
            if(selectedEVM === 0)
            {
                templateObj.$.ti_widget_droplist_config_SW0.selectedValue = data[0].sw0;
                templateObj.$.ti_widget_droplist_config_SW1.selectedValue = data[0].sw1;
            }
            templateObj.$.ti_widget_droplist_selectedEVM.selectedValue = 0;
            Utilities.fireEvent('select-evm', templateObj.$.ti_widget_droplist_selectedEVM.selectedValue, "plots.js");
        }
    };
    gc.databind.registry.getBinding('widget.ti_widget_droplist_E1_SW0.selectedValue').addChangedListener(swE1);
    gc.databind.registry.getBinding('widget.ti_widget_droplist_E1_SW1.selectedValue').addChangedListener(swE1);
    
    const swE2 = {
        onValueChanged: function() {
            data[1].sw0 = templateObj.$.ti_widget_droplist_E2_SW0.selectedValue;
            data[1].sw1 = templateObj.$.ti_widget_droplist_E2_SW1.selectedValue;
            templateObj.$.ti_widget_droplist_selectedEVM.selectedValue = 1;
            if(selectedEVM === 1)
            {
                templateObj.$.ti_widget_droplist_config_SW0.selectedValue = data[1].sw0;
                templateObj.$.ti_widget_droplist_config_SW1.selectedValue = data[1].sw1;
            }
            Utilities.fireEvent('select-evm', templateObj.$.ti_widget_droplist_selectedEVM.selectedValue, "plots.js");
        }
    };
    gc.databind.registry.getBinding('widget.ti_widget_droplist_E2_SW0.selectedValue').addChangedListener(swE2);
    gc.databind.registry.getBinding('widget.ti_widget_droplist_E2_SW1.selectedValue').addChangedListener(swE2);
    
    const swE3 = {
        onValueChanged: function() {
            data[2].sw0 = templateObj.$.ti_widget_droplist_E3_SW0.selectedValue;
            data[2].sw1 = templateObj.$.ti_widget_droplist_E3_SW1.selectedValue;
            templateObj.$.ti_widget_droplist_selectedEVM.selectedValue = 2;
            if(selectedEVM === 2)
            {
                templateObj.$.ti_widget_droplist_config_SW0.selectedValue = data[2].sw0;
                templateObj.$.ti_widget_droplist_config_SW1.selectedValue = data[2].sw1;
            }
            Utilities.fireEvent('select-evm', templateObj.$.ti_widget_droplist_selectedEVM.selectedValue, "plots.js");
        }
    };
    gc.databind.registry.getBinding('widget.ti_widget_droplist_E3_SW0.selectedValue').addChangedListener(swE3);
    gc.databind.registry.getBinding('widget.ti_widget_droplist_E3_SW1.selectedValue').addChangedListener(swE3);
    
    const swE4 = {
        onValueChanged: function() {
            data[3].sw0 = templateObj.$.ti_widget_droplist_E4_SW0.selectedValue;
            data[3].sw1 = templateObj.$.ti_widget_droplist_E4_SW1.selectedValue;
            templateObj.$.ti_widget_droplist_selectedEVM.selectedValue = 3;
            if(selectedEVM === 3)
            {
                templateObj.$.ti_widget_droplist_config_SW0.selectedValue = data[3].sw0;
                templateObj.$.ti_widget_droplist_config_SW1.selectedValue = data[3].sw1;
            }
            Utilities.fireEvent('select-evm', templateObj.$.ti_widget_droplist_selectedEVM.selectedValue, "plots.js");
        }
    };
    gc.databind.registry.getBinding('widget.ti_widget_droplist_E4_SW0.selectedValue').addChangedListener(swE4);
    gc.databind.registry.getBinding('widget.ti_widget_droplist_E4_SW1.selectedValue').addChangedListener(swE4);
    
            
    // let myBinding = gc.databind.registry.getBinding('widget.ti_widget_numberbox_collect_delay.value');
    // myBinding.addChangedListener(
    // {
    //     onValueChanged: function(oldValue, newValue)
    //     {
    //         if(templateObj.$.ti_widget_checkbox_autodelay.checked)
    //         {
    //             this.fire('update-delay', Math.round(newValue*1000));
    //         }
    //     }
    // });

    // gc.databind.registry.getBinding('widget.ti_widget_numberbox_collect_delay.value').addChangedListener(
    // {
    //     onValueChanged: function(oldValue, newValue)
    //     {
    //         if(templateObj.$.ti_widget_checkbox_autodelay.checked)
    //         {
    //             this.fire('update-delay', Math.round(newValue*1000));
    //         }
    //     }
    // });
	
});



/*
*  Boilerplate code for creating custom actions
*/
document.addEventListener('gc-nav-ready', function() 
{
    /* 
	*   Add custom actions for menu items using the following api:
	*
    *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
	*
	*   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
	*   param runable - function that performs the custom action.
	*   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
	*   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
    */
	
	// For example,
	// gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });
	
	// Alternatively, to programmatically disable a menu action at any time use:
	// gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction'); 
});

/*
*  Boilerplate code for working with components in the application gist
*/
/**/

var initComplete = false;
//var templateObj;

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');

    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
	templateObj.addEventListener('dom-change',function(){
	    if (initComplete) return;
	    this.async(function(){
    	    initComplete = true;
    	    console.log("Application template has been stamped.");
  	        // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
  	        // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId
  	        
            templateObj.$.ti_widget_button_clear_VSHUNT.addEventListener("click", function(event) {
                this.fire('clear-vshunt');
            });
            templateObj.$.ti_widget_button_save_VSHUNT.addEventListener("click", function (event) 
            {
                savePlots(PLOT_VSHUNT, "VSHUNT_Results.csv");
            });
            templateObj.$.ti_widget_button_clear_VBUS.addEventListener("click", function(event) {
                this.fire('clear-vbus');
            });
            templateObj.$.ti_widget_button_save_VBUS.addEventListener("click", function (event) 
            {
                savePlots(PLOT_VBUS, "VBUS_Results.csv");
            });
            templateObj.$.ti_widget_button_clear_DIETEMP.addEventListener("click", function(event) {
                this.fire('clear-dietemp');
            });
            templateObj.$.ti_widget_button_save_DIETEMP.addEventListener("click", function (event) 
            {
                savePlots(PLOT_DIETEMP, "DIETEMP_Results.csv");
            });
            templateObj.$.ti_widget_button_clear_CURRENT.addEventListener("click", function(event) {
                this.fire('clear-current');
            });
            templateObj.$.ti_widget_button_save_CURRENT.addEventListener("click", function (event) 
            {
                savePlots(PLOT_CURRENT, "CURRENT_Results.csv");
            });
            templateObj.$.ti_widget_button_clear_POWER.addEventListener("click", function(event) {
                this.fire('clear-power');
            });
            templateObj.$.ti_widget_button_save_POWER.addEventListener("click", function (event) 
            {
                savePlots(PLOT_POWER, "POWER_Results.csv");
            });
            templateObj.$.ti_widget_button_clear_ENERGY.addEventListener("click", function(event) {
                this.fire('clear-energy');
            });
            templateObj.$.ti_widget_button_save_ENERGY.addEventListener("click", function (event) 
            {
                savePlots(PLOT_ENERGY, "ENERGY_Results.csv");
            });
            templateObj.$.ti_widget_button_clear_CHARGE.addEventListener("click", function(event) {
                this.fire('clear-charge');
            });
            templateObj.$.ti_widget_button_save_CHARGE.addEventListener("click", function (event) 
            {
                savePlots(PLOT_CHARGE, "CHARGE_Results.csv");
            });
            templateObj.$.ti_widget_button_clear_all.addEventListener("click", function(event) {
                this.fire('clear-all-plots');
                numSamplesCollected = 0;
            });
            templateObj.$.ti_widget_numberbox_samples_all.addEventListener("changed", function(event) {
                numSamplesCollected = 0;
            });
            templateObj.$.ti_widget_button_save_all.addEventListener("click", function (event) 
            {
                let plotFlags = 0;
                
                plotFlags |= templateObj.$.ti_widget_checkbox_VSHUNT.checked ? 1 : 0;
                plotFlags <<= 1;
                plotFlags |= templateObj.$.ti_widget_checkbox_VBUS.checked ? 1 : 0;
                plotFlags <<= 1;
                plotFlags |= templateObj.$.ti_widget_checkbox_DIETEMP.checked ? 1 : 0;
                plotFlags <<= 1;
                plotFlags |= templateObj.$.ti_widget_checkbox_CURRENT.checked ? 1 : 0;
                plotFlags <<= 1;
                plotFlags |= templateObj.$.ti_widget_checkbox_POWER.checked ? 1 : 0;
                plotFlags <<= 1;
                plotFlags |= templateObj.$.ti_widget_checkbox_ENERGY.checked ? 1 : 0;
                plotFlags <<= 1;
                plotFlags |= templateObj.$.ti_widget_checkbox_CHARGE.checked ? 1 : 0;
                
                savePlots(plotFlags, "All_Results.csv");
            });
            templateObj.$.ti_widget_button_collect_start.addEventListener("click", function(event) {
                this.fire('collect-data-plus');
            });
            templateObj.$.ti_widget_button_collect_stop.addEventListener("click", function(event) {
                this.fire('abort-collect');
            });
            templateObj.$.ti_widget_checkbox_autodelay.addEventListener("changed", function(event) {
                if(templateObj.$.ti_widget_checkbox_autodelay.checked)
                {
                    templateObj.$.ti_widget_numberbox_collect_delay.value = delayTime_Used;
                    this.fire('update-delay', Math.round(delayTime_Used*1000));
                }
            });
            templateObj.$.ti_widget_numberbox_collect_delay.addEventListener("changed", function(event) {
                this.fire('update-delay', Math.round(templateObj.$.ti_widget_numberbox_collect_delay.value*1000));
            });
            templateObj.$.ti_widget_checkbox_vshunt_manual.addEventListener("changed", function(event) {
                if(templateObj.$.ti_widget_checkbox_vshunt_manual.checked)
                {
                    let numEVMs = templateObj.$.ti_widget_droplist_numEVMs.selectedValue;
                    
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_vshuntMax.value))
                    {
                        let max
                        let maxTemp
                        max = templateObj.$.ti_widget_scalargraph_VSHUNT.series_0_values.reduce(function(a,b){return Math.max(a, b);});
                        if(numEVMs > 1)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_VSHUNT.series_1_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 2)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_VSHUNT.series_2_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 3)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_VSHUNT.series_3_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        templateObj.$.ti_widget_textbox_vshuntMax.value = max;
                    }
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_vshuntMin.value))
                    {
                        let min
                        let minTemp
                        min = templateObj.$.ti_widget_scalargraph_VSHUNT.series_0_values.reduce(function(a,b){return Math.min(a, b);});
                        if(numEVMs > 1)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_VSHUNT.series_1_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 2)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_VSHUNT.series_2_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 3)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_VSHUNT.series_3_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        templateObj.$.ti_widget_textbox_vshuntMin.value = min;
                    }
                }
            });
            templateObj.$.ti_widget_checkbox_vbus_manual.addEventListener("changed", function(event) {
                if(templateObj.$.ti_widget_checkbox_vbus_manual.checked)
                {
                    let numEVMs = templateObj.$.ti_widget_droplist_numEVMs.selectedValue;
                    
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_vbusMax.value))
                    {
                        let max
                        let maxTemp
                        max = templateObj.$.ti_widget_scalargraph_VBUS.series_0_values.reduce(function(a,b){return Math.max(a, b);});
                        if(numEVMs > 1)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_VBUS.series_1_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 2)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_VBUS.series_2_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 3)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_VBUS.series_3_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        templateObj.$.ti_widget_textbox_vbusMax.value = max;
                    }
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_vbusMin.value))
                    {
                        let min
                        let minTemp
                        min = templateObj.$.ti_widget_scalargraph_VBUS.series_0_values.reduce(function(a,b){return Math.min(a, b);});
                        if(numEVMs > 1)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_VBUS.series_1_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 2)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_VBUS.series_2_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 3)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_VBUS.series_3_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        templateObj.$.ti_widget_textbox_vbusMin.value = min;
                    }
                }
            });
            templateObj.$.ti_widget_checkbox_dietemp_manual.addEventListener("changed", function(event) {
                if(templateObj.$.ti_widget_checkbox_dietemp_manual.checked)
                {
                    let numEVMs = templateObj.$.ti_widget_droplist_numEVMs.selectedValue;
                    
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_dietempMax.value))
                    {
                        let max
                        let maxTemp
                        max = templateObj.$.ti_widget_scalargraph_DIETEMP.series_0_values.reduce(function(a,b){return Math.max(a, b);});
                        if(numEVMs > 1)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_DIETEMP.series_1_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 2)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_DIETEMP.series_2_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 3)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_DIETEMP.series_3_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        templateObj.$.ti_widget_textbox_dietempMax.value = max;
                    }
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_dietempMin.value))
                    {
                        let min
                        let minTemp
                        min = templateObj.$.ti_widget_scalargraph_DIETEMP.series_0_values.reduce(function(a,b){return Math.min(a, b);});
                        if(numEVMs > 1)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_DIETEMP.series_1_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 2)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_DIETEMP.series_2_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 3)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_DIETEMP.series_3_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        templateObj.$.ti_widget_textbox_dietempMin.value = min;
                    }
                }
            });
            templateObj.$.ti_widget_checkbox_current_manual.addEventListener("changed", function(event) {
                if(templateObj.$.ti_widget_checkbox_current_manual.checked)
                {
                    let numEVMs = templateObj.$.ti_widget_droplist_numEVMs.selectedValue;
                    
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_currentMax.value))
                    {
                        let max
                        let maxTemp
                        max = templateObj.$.ti_widget_scalargraph_CURRENT.series_0_values.reduce(function(a,b){return Math.max(a, b);});
                        if(numEVMs > 1)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_CURRENT.series_1_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 2)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_CURRENT.series_2_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 3)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_CURRENT.series_3_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        templateObj.$.ti_widget_textbox_currentMax.value = max;
                    }
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_currentMin.value))
                    {
                        let min
                        let minTemp
                        min = templateObj.$.ti_widget_scalargraph_CURRENT.series_0_values.reduce(function(a,b){return Math.min(a, b);});
                        if(numEVMs > 1)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_CURRENT.series_1_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 2)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_CURRENT.series_2_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 3)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_CURRENT.series_3_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        templateObj.$.ti_widget_textbox_currentMin.value = min;
                    }
                }
            });
            templateObj.$.ti_widget_checkbox_power_manual.addEventListener("changed", function(event) {
                if(templateObj.$.ti_widget_checkbox_power_manual.checked)
                {
                    let numEVMs = templateObj.$.ti_widget_droplist_numEVMs.selectedValue;
                    
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_powerMax.value))
                    {
                        let max
                        let maxTemp
                        max = templateObj.$.ti_widget_scalargraph_POWER.series_0_values.reduce(function(a,b){return Math.max(a, b);});
                        if(numEVMs > 1)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_POWER.series_1_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 2)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_POWER.series_2_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 3)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_POWER.series_3_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        templateObj.$.ti_widget_textbox_powerMax.value = max;
                    }
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_powerMin.value))
                    {
                        let min
                        let minTemp
                        min = templateObj.$.ti_widget_scalargraph_POWER.series_0_values.reduce(function(a,b){return Math.min(a, b);});
                        if(numEVMs > 1)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_POWER.series_1_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 2)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_POWER.series_2_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 3)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_POWER.series_3_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        templateObj.$.ti_widget_textbox_powerMin.value = min;
                    }
                }
            });
            templateObj.$.ti_widget_checkbox_energy_manual.addEventListener("changed", function(event) {
                if(templateObj.$.ti_widget_checkbox_energy_manual.checked)
                {
                    let numEVMs = templateObj.$.ti_widget_droplist_numEVMs.selectedValue;
                    
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_energyMax.value))
                    {
                        let max
                        let maxTemp
                        max = templateObj.$.ti_widget_scalargraph_ENERGY.series_0_values.reduce(function(a,b){return Math.max(a, b);});
                        if(numEVMs > 1)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_ENERGY.series_1_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 2)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_ENERGY.series_2_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 3)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_ENERGY.series_3_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        templateObj.$.ti_widget_textbox_energyMax.value = max;
                    }
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_energyMin.value))
                    {
                        let min
                        let minTemp
                        min = templateObj.$.ti_widget_scalargraph_ENERGY.series_0_values.reduce(function(a,b){return Math.min(a, b);});
                        if(numEVMs > 1)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_ENERGY.series_1_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 2)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_ENERGY.series_2_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 3)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_ENERGY.series_3_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        templateObj.$.ti_widget_textbox_energyMin.value = min;
                    }
                }
            });
            templateObj.$.ti_widget_checkbox_charge_manual.addEventListener("changed", function(event) {
                if(templateObj.$.ti_widget_checkbox_charge_manual.checked)
                {
                    let numEVMs = templateObj.$.ti_widget_droplist_numEVMs.selectedValue;
                    
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_chargeMax.value))
                    {
                        let max
                        let maxTemp
                        max = templateObj.$.ti_widget_scalargraph_CHARGE.series_0_values.reduce(function(a,b){return Math.max(a, b);});
                        if(numEVMs > 1)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_CHARGE.series_1_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 2)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_CHARGE.series_2_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        if(numEVMs > 3)
                        {
                            maxTemp = templateObj.$.ti_widget_scalargraph_CHARGE.series_3_values.reduce(function(a,b){return Math.max(a, b);});
                            max = Math.max(max, maxTemp);
                        }
                        templateObj.$.ti_widget_textbox_chargeMax.value = max;
                    }
                    if(isEmptyOrSpaces(templateObj.$.ti_widget_textbox_chargeMin.value))
                    {
                        let min
                        let minTemp
                        min = templateObj.$.ti_widget_scalargraph_CHARGE.series_0_values.reduce(function(a,b){return Math.min(a, b);});
                        if(numEVMs > 1)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_CHARGE.series_1_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 2)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_CHARGE.series_2_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        if(numEVMs > 3)
                        {
                            minTemp = templateObj.$.ti_widget_scalargraph_CHARGE.series_3_values.reduce(function(a,b){return Math.min(a, b);});
                            min = Math.min(min, minTemp);
                        }
                        templateObj.$.ti_widget_textbox_chargeMin.value = min;
                    }
                }
            });
            templateObj.$.ti_widget_droplist_dietemp_unit.addEventListener("selected", function(event) {
                let numEVMs = templateObj.$.ti_widget_droplist_numEVMs.selectedValue;
                let strData0Ref = templateObj.$.ti_widget_scalargraph_DIETEMP.series_0_values;
                let strData1Ref = templateObj.$.ti_widget_scalargraph_DIETEMP.series_1_values;
                let strData2Ref = templateObj.$.ti_widget_scalargraph_DIETEMP.series_2_values;
                let strData3Ref = templateObj.$.ti_widget_scalargraph_DIETEMP.series_3_values;
                let data0Last;
                let data1Last;
                let data2Last;
                let data3Last;
                
                //Convert data. Offset by one value so that we can force a visual update.
                if(ti_widget_droplist_dietemp_unit.selectedValue === 2)
                {
                    data0Last = (strData0Ref[strData0Ref.length-1] * (9/5)) + 32;
                    if(numEVMs > 1)
                    {
                        data1Last = (strData1Ref[strData1Ref.length-1] * (9/5)) + 32;
                    }
                    if(numEVMs > 2)
                    {
                        data2Last = (strData2Ref[strData2Ref.length-1] * (9/5)) + 32;
                    }
                    if(numEVMs > 3)
                    {
                        data3Last = (strData3Ref[strData3Ref.length-1] * (9/5)) + 32;
                    }
                    for (let i = strData0Ref.length-2; i >= 0; i--) 
                    {
                        strData0Ref[i+1] = (strData0Ref[i] * (9/5)) + 32;
                        if(numEVMs > 1)
                        {
                            strData1Ref[i+1] = (strData1Ref[i] * (9/5)) + 32;
                        }
                        if(numEVMs > 2)
                        {
                            strData2Ref[i+1] = (strData2Ref[i] * (9/5)) + 32;
                        }
                        if(numEVMs > 3)
                        {
                            strData3Ref[i+1] = (strData3Ref[i] * (9/5)) + 32;
                        }
                    }
                    //temporary values that will get pushed out of the arrays
                    strData0Ref[0] = (strData0Ref[0] * (9/5)) + 32;
                    if(numEVMs > 1)
                    {
                        strData1Ref[0] = (strData1Ref[0] * (9/5)) + 32;
                    }
                    if(numEVMs > 2)
                    {
                        strData2Ref[0] = (strData2Ref[0] * (9/5)) + 32;
                    }
                    if(numEVMs > 3)
                    {
                        strData3Ref[0] = (strData3Ref[0] * (9/5)) + 32;
                    }
                }
                else
                {
                    data0Last = (strData0Ref[strData0Ref.length-1] - 32) * (5/9);
                    if(numEVMs > 1)
                    {
                        data1Last = (strData1Ref[strData1Ref.length-1] - 32) * (5/9);
                    }
                    if(numEVMs > 2)
                    {
                        data2Last = (strData2Ref[strData2Ref.length-1] - 32) * (5/9);
                    }
                    if(numEVMs > 3)
                    {
                        data3Last = (strData3Ref[strData3Ref.length-1] - 32) * (5/9);
                    }
                    for (let i = strData0Ref.length-2; i >= 0; i--)
                    {
                        strData0Ref[i+1] = (strData0Ref[i] - 32) * (5/9);
                        if(numEVMs > 1)
                        {
                            strData1Ref[i+1] = (strData1Ref[i] - 32) * (5/9);
                        }
                        if(numEVMs > 2)
                        {
                            strData2Ref[i+1] = (strData2Ref[i] - 32) * (5/9);
                        }
                        if(numEVMs > 3)
                        {
                            strData3Ref[i+1] = (strData3Ref[i] - 32) * (5/9);
                        }
                    }
                    //temporary values that will get pushed out of the arrays
                    strData0Ref[0] = (strData0Ref[0] - 32) * (5/9);
                    if(numEVMs > 1)
                    {
                        strData1Ref[0] = (strData1Ref[0] - 32) * (5/9);
                    }
                    if(numEVMs > 2)
                    {
                        strData2Ref[0] = (strData2Ref[0] - 32) * (5/9);
                    }
                    if(numEVMs > 3)
                    {
                        strData3Ref[0] = (strData3Ref[0] - 32) * (5/9);
                    }
                }
                
                //update scale
                if(templateObj.$.ti_widget_checkbox_dietemp_manual.checked)
                {
                    if(ti_widget_droplist_dietemp_unit.selectedValue === 2)
                    {
                        templateObj.$.ti_widget_textbox_dietempMax.value = roundTo((templateObj.$.ti_widget_textbox_dietempMax.value * (9/5)) + 32,13);
                        templateObj.$.ti_widget_textbox_dietempMin.value = roundTo((templateObj.$.ti_widget_textbox_dietempMin.value * (9/5)) + 32,13);
                    }
                    else
                    {
                        templateObj.$.ti_widget_textbox_dietempMax.value = roundTo((templateObj.$.ti_widget_textbox_dietempMax.value - 32) * (5/9),13);
                        templateObj.$.ti_widget_textbox_dietempMin.value = roundTo((templateObj.$.ti_widget_textbox_dietempMin.value - 32) * (5/9),13);
                    }
                }
                
                //put the last data sample in manally to force visual update
		        if(templateObj.$.ti_widget_scalargraph_DIETEMP.value_0 === data0Last)
			    {
			        templateObj.$.ti_widget_scalargraph_DIETEMP.value_0Changed();
			    }
			    else
			    {
			        templateObj.$.ti_widget_scalargraph_DIETEMP.value_0 = data0Last;
			    }
        				    
                if(numEVMs > 1)
                {
    		        if(templateObj.$.ti_widget_scalargraph_DIETEMP.value_1 === data1Last)
    			    {
    			        templateObj.$.ti_widget_scalargraph_DIETEMP.value_1Changed();
    			    }
    			    else
    			    {
    			        templateObj.$.ti_widget_scalargraph_DIETEMP.value_1 = data1Last;
    			    }
                }
                if(numEVMs > 2)
                {
    		        if(templateObj.$.ti_widget_scalargraph_DIETEMP.value_2 === data2Last)
    			    {
    			        templateObj.$.ti_widget_scalargraph_DIETEMP.value_2Changed();
    			    }
    			    else
    			    {
    			        templateObj.$.ti_widget_scalargraph_DIETEMP.value_2 = data2Last;
    			    }
                }
                if(numEVMs > 3)
                {
    		        if(templateObj.$.ti_widget_scalargraph_DIETEMP.value_3 === data3Last)
    			    {
    			        templateObj.$.ti_widget_scalargraph_DIETEMP.value_3Changed();
    			    }
    			    else
    			    {
    			        templateObj.$.ti_widget_scalargraph_DIETEMP.value_3 = data3Last;
    			    }
                }
                changingUnits = 0;
            });
            
        },1);

	});
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded',init.bind(this))
}


